#line 1

#define MAX_LIGHTS 8

#define LIGHT_OMNI 0
#define LIGHT_OMNINORMAL 1
#define LIGHT_SPOT 2
#define LIGHT_SPOTNORMAL 3
#define LIGHT_SHADOW 4

// lights
uniform int numLights;
uniform vec3 lightPos[MAX_LIGHTS];
uniform vec3 lightColor[MAX_LIGHTS];
uniform vec3 lightParams[MAX_LIGHTS];
uniform mat3 lightDir[MAX_LIGHTS];

#ifdef MODEL_EFFECTS
// shadows
uniform bool modelShadow;
uniform vec3 shadowColor;

bool inShadow;

void ProcessShadow(vec3 lightPos, vec3 lightSize, mat3 lightDir)
{
  if (!inShadow) {
    vec3 delta = lightPos - inPosition.xyz;
    float dist = dot(delta, lightDir[0]);
    if (dist < -lightSize.x || dist > lightSize.x) {
      return;
    }

    dist = dot(delta, lightDir[1]);
    if (dist < -lightSize.y || dist > lightSize.y) {
      return;
    }

    dist = dot(delta, lightDir[2]);
    if (dist < -lightSize.z || dist > lightSize.z) {
      return;
    }

    inShadow = true;
  }
}
#endif  // MODEL_EFFECTS

void ProcessLight(vec3 lightPos, vec3 lightColor, vec3 lightParams, mat3 lightDir)
{
  int type = int(lightParams.x);

#ifdef MODEL_EFFECTS
  // special case for shadow box
  if (type == LIGHT_SHADOW) {
    ProcessShadow(lightPos, lightColor, lightDir);

  // other lights
  } else
#endif  // MODEL_EFFECTS
  {
    float reach = lightParams.y;
    vec3 delta = lightPos - inPosition.xyz;
    float dist = dot(delta, delta);
    if (dist < reach) {
      float scale = 1.0 - (dist / reach);
      dist = sqrt(dist);

      // calc angle from normal if needed
      if (type == LIGHT_OMNINORMAL || type == LIGHT_SPOTNORMAL) {
        float ang = dot(delta, inNormal);
        ang = clamp(ang / dist, 0.0, 1.0);
        scale *= ang;
      }

      // calc cone adjustment if needed
      if (type == LIGHT_SPOT || type == LIGHT_SPOTNORMAL) {
        float conemul = lightParams.z;
        float cone = -dot(lightDir[2], delta) / dist - 1.0;
        cone = cone * conemul + 1.0;
        cone = clamp(cone, 0.0, 1.0);
        scale *= cone;
      }

      varColor.rgb += mix(vec3(0.0), lightColor, scale);
    }
  }
}

void ProcessLights(void)
{
#ifdef MODEL_EFFECTS
  inShadow = modelShadow;
#endif  // MODEL_EFFECTS

  if (numLights > 0) {
    ProcessLight(lightPos[0], lightColor[0], lightParams[0], lightDir[0]);
  }
  if (numLights > 1) {
    ProcessLight(lightPos[1], lightColor[1], lightParams[1], lightDir[1]);
  }
  if (numLights > 2) {
    ProcessLight(lightPos[2], lightColor[2], lightParams[2], lightDir[2]);
  }
  if (numLights > 3) {
    ProcessLight(lightPos[3], lightColor[3], lightParams[3], lightDir[3]);
  }
  if (numLights > 4) {
    ProcessLight(lightPos[4], lightColor[4], lightParams[4], lightDir[4]);
  }
  if (numLights > 5) {
    ProcessLight(lightPos[5], lightColor[5], lightParams[5], lightDir[5]);
  }
  if (numLights > 6) {
    ProcessLight(lightPos[6], lightColor[6], lightParams[6], lightDir[6]);
  }
  if (numLights > 7) {
    ProcessLight(lightPos[7], lightColor[7], lightParams[7], lightDir[7]);
  }

#ifdef MODEL_EFFECTS
  if (inShadow) {
    varColor.rgb += shadowColor;
  }
#endif  // MODEL_EFFECTS
}
